@@ -346,7 +346,7 @@ def convolution_kernel_gaussian(fwhm_wave, target_fwhm, target_psf,
 
     return conv_kernel
 
-def cube_convolve(data, kernel, variance=None, fft=True):
+def cube_convolve(data, kernel, variance=None, fft=True, fill_value=np.nan):
     """Convolve a 3D datacube
 
     Args:
@@ -379,10 +379,11 @@ def cube_convolve(data, kernel, variance=None, fft=True):
                                       norm_kernel[i, :, :],
                                       allow_huge=True,
                                       psf_pad=True,
-                                      fft_pad=False,
+                                      fft_pad=True,
                                       boundary='fill',
-                                      fill_value=0,
-                                      normalize_kernel=False)
+                                      fill_value=fill_value,
+                                      normalize_kernel=True,
+                                      preserve_nan=True)
 
     if variance is not None:
         for i in range(variance.shape[0]):
@@ -391,10 +392,11 @@ def cube_convolve(data, kernel, variance=None, fft=True):
                                               var_kernel[i, :, :],
                                               allow_huge=True,
                                               psf_pad=True,
-                                              fft_pad=False,
+                                              fft_pad=True,
                                               boundary='fill',
-                                              fill_value=0,
-                                              normalize_kernel=False)
+                                              fill_value=fill_value,
+                                              normalize_kernel=False,
+                                              preserve_nan=True)
     # else:
     #     print("Convolution using 3D convolve in astropy")
     #     data = conv_function(data, norm_kernel, allow_huge=True,
@@ -454,6 +456,18 @@ def cube_kernel(shape, wave, input_fwhm,  target_fwhm,
                        nmoffat=target_nmoffat, scale=scale)
 
     print('Creating the convolution kernel')
+    print("-----------------------------------------------------------")
+    print(f"Input function = {input_function}, Input FWHM = {input_fwhm}")
+    if input_function == "moffat":
+        print(f"Input N_moffat = {input_nmoffat}")
+    if b == 0:
+        print(f"No Wavelength variation (b=0)")
+    else:
+        print(f"Wavelength variation given by lambda0={lambda0} and b={b}")
+    print("-----------------------------------------------------------")
+    print(f"Target function = {target_function}, target FWHM = {target_fwhm}")
+    if target_function == "moffat":
+        print(f"Target N_moffat = {target_nmoffat}")
     if compute_kernel == 'pypher':
         print('Building the convolution kernel via pypher')
         kernel = convolution_kernel(original_psf, target_psf, scale=0.2)
