@@ -21,11 +21,13 @@ import numpy as np
 # pymusepipe modules
 from . import util_pipe as upipe
 from .create_sof import SofPipe
-from .align_pipe import create_offset_table
+from .align_pipe import create_offset_table, AlignMuseField
 from . import musepipe
 from .mpdaf_pipe import MuseSkyContinuum, MuseFilter
-from .config_pipe import mjd_names,get_suffix_product
-from .config_pipe import dict_recipes_per_num, dict_recipes_per_name
+from .config_pipe import (mjd_names, get_suffix_product, dict_default_for_recipes,
+                          dict_recipes_per_num, dict_recipes_per_name,
+                          dict_files_iexpo_products, dict_files_products,
+                          dict_products_scipost)
 
 try :
     import astropy as apy
@@ -33,37 +35,6 @@ try :
 except ImportError :
     raise Exception("astropy is required for this module")
 
-# =======================================================
-# List of recipes
-# =======================================================
-list_recipes = ['bias', 'flat', 'wave', 'lsf', 
-        'twilight', 'scibasic_all', 'std', 'sky']
-        
-dict_files_iexpo_products = {
-        'ALIGN': ['SOURCE_LIST']
-        }
-
-dict_files_products = {
-        'STD': ['DATACUBE_STD', 'STD_FLUXES', 
-            'STD_RESPONSE', 'STD_TELLURIC'],
-        'TWILIGHT': ['DATACUBE_SKYFLAT', 'TWILIGHT_CUBE'],
-        'SKY': ['SKY_MASK', 'SKY_IMAGE', 'SKY_LINES', 'SKY_SPECTRUM', 
-            'SKY_CONTINUUM'],
-        'ALIGN': ['OFFSET_LIST']
-        }
-
-dict_products_scipost = {
-        'cube': ['DATACUBE_FINAL', 'IMAGE_FOV'],
-        'individual': ['PIXTABLE_REDUCED'],
-        'stacked': ['OBJECT_RESAMPLED'],
-        'positioned': ['PIXTABLE_POSITIONED'],
-        'combined': ['PIXTABLE_COMBINED'],
-        'skymodel': ['SKY_MASK', 'SKY_SPECTRUM', 
-            'SKY_LINES', 'SKY_IMAGE'],
-        'raman': ['RAMAN_IMAGES'],
-        'autocal': ['AUTOCAL_FACTORS']
-        }
-
 # =======================================================
 # Few useful functions
 # =======================================================
@@ -91,7 +62,7 @@ def print_my_function_name(f):
     """
     @functools.wraps(f)
     def wrapped(*myargs, **mykwargs):
-        upipe.print_info(f.__name__)
+        upipe.print_info("################   " + f.__name__ + "   ################")
         return f(*myargs, **mykwargs)
     return wrapped
 
@@ -106,8 +77,8 @@ def _get_combine_products(filter_list='white', prefix_all=""):
     for prod in dict_products_scipost['cube']:
         if prod == "IMAGE_FOV":
             for i, value in enumerate(filter_list.split(','), start=1):
-                suffix_products.append("_{0:04d}".format(i))
-                suffix_prefinalnames.append("_{0}".format(value))
+                suffix_products.append(f"_{i:04d}")
+                suffix_prefinalnames.append(f"_{value}")
                 name_products.append(prod)
                 prefix_products.append(prefix_all)
         else :
@@ -129,7 +100,6 @@ class PipePrep(SofPipe) :
         """
         SofPipe.__init__(self)
 #        super(PipePrep, self).__init__()
-        self.list_recipes = deepcopy(list_recipes)
         self.first_recipe = first_recipe
         if last_recipe is None:
             self.last_recipe = np.max(list(dict_recipes_per_num.keys()))
@@ -172,15 +142,14 @@ class PipePrep(SofPipe) :
     def print_recipes():
         """Printing the list of recipes
         """
-        # for recipe in self.list_recipes:
         upipe.print_info("=============================================")
         upipe.print_info("The dictionary of recipes which can be run is")
-        for key in dict_recipes_num:
-            print("{0}: {1}".format(key, dict_recipes_num[key]))
+        for key in dict_recipes_per_num:
+            print("{0}: {1}".format(key, dict_recipes_per_num[key]))
         upipe.print_info("=============================================")
 
     @print_my_function_name
-    def run_recipes(self, fraction=0.8, skymethod="model", illum=True, **kwargs):
+    def run_recipes(self, **kwargs):
         """Running all recipes in one shot
 
         Input
@@ -192,13 +161,35 @@ class PipePrep(SofPipe) :
             Default is True (use illumination during twilight calibration)
         skymethod: str
             Default is "model".
+        filter_for_alignment: str
+            Default is defined in config_pipe
+        line: str
+            Default is None as defined in config_pipe
+        lambda_window: float
+            Default is 10.0 as defined in config_pipe
         """
+        class DefVal(object):
+            def __init__(self):
+                pass
+
+        defval = DefVal()
+        for key in dict_default_for_recipes:
+            setattr(defval, key, kwargs.get(key, dict_default_for_recipes[key]))
+
+        # We overwrite filter_for_alignment as it can be already set
+        defval.filter_for_alignment = kwargs.pop("filter_for_alignment", self.filter_for_alignment)
+
         # Dictionary of arguments for each recipe
-        default_dict_kwargs_recipes = {'twilight': {'illum': illum},
-                             'scibasic_all': {'illum': illum},
-                             'sky': {'fraction': fraction},
-                             'prep_align': {'skymethod': skymethod},
-                             'scipost_per_expo': {'skymethod': skymethod}
+        default_dict_kwargs_recipes = {'twilight': {'illum': defval.illum},
+                             'scibasic_all': {'illum': defval.illum},
+                             'sky': {'fraction': defval.fraction},
+                             'prep_align': {'skymethod': defval.skymethod, 
+                                            'filter_for_alignment': defval.filter_for_alignment,
+                                            'line': defval.line,
+                                            'lambda_window': defval.lambda_window
+                                            },
+                             'scipost_per_expo': {'skymethod': defval.skymethod},
+                             'scipost': {'skymethod': defval.skymethod}
                              }
 
         dict_kwargs_recipes = kwargs.pop("param_recipes", {})
@@ -512,16 +503,19 @@ class PipePrep(SofPipe) :
         self.goto_prevfolder(addtolog=True)
 
     @print_my_function_name
-    def run_scibasic_all(self, list_object=['OBJECT', 'SKY', 'STD'], tpl="ALL", illum=True):
+    def run_scibasic_all(self, list_object=['OBJECT', 'SKY', 'STD'], tpl="ALL",
+                         illum=True, **kwargs):
         """Running scibasic for all objects in list_object
         Making different sof for each category
         """
         for expotype in list_object:
             sof_filename = 'scibasic_{0}'.format(expotype.lower())
-            self.run_scibasic(sof_filename=sof_filename, expotype=expotype, tpl=tpl, illum=illum)
+            self.run_scibasic(sof_filename=sof_filename, expotype=expotype,
+                              tpl=tpl, illum=illum, **kwargs)
 
     @print_my_function_name
-    def run_scibasic(self, sof_filename='scibasic', expotype="OBJECT", tpl="ALL", illum=True):
+    def run_scibasic(self, sof_filename='scibasic', expotype="OBJECT", tpl="ALL", illum=True,
+                     update=True, overwrite=True):
         """Reducing the files of a certain category and creating the PIXTABLES
         Will run the esorex muse_scibasic 
 
@@ -546,7 +540,7 @@ class PipePrep(SofPipe) :
 
         # Create the dictionary for the LSF including
         # the list of files to be processed for one MASTER Flat
-        for gtable in tpl_gtable.groups:
+        for ntable, gtable in enumerate(tpl_gtable.groups):
             self._add_calib_to_sofdict("BADPIX_TABLE", reset=True)
             self._add_calib_to_sofdict("LINE_CATALOG")
             # extract the tpl (string) and mean mjd (float) 
@@ -581,14 +575,19 @@ class PipePrep(SofPipe) :
             # Write the Processed files Table and save it
             # Note: always overwrite the table so that a fresh numbering is done
             gtable['iexpo'] = list_expo
-            self.save_expo_table(expotype, gtable, "processed", aggregate=False,
-                                 update=False, overwrite=True)
+            if ntable == 0:
+                self.save_expo_table(expotype, gtable, "processed", aggregate=False,
+                                     update=update, overwrite=overwrite)
+            else:
+                self.save_expo_table(expotype, gtable, "processed", aggregate=False,
+                                     update=update, overwrite=False)
 
         # Go back to original folder
         self.goto_prevfolder(addtolog=True)
 
     @print_my_function_name
-    def run_standard(self, sof_filename='standard', tpl="ALL", update=None):
+    def run_standard(self, sof_filename='standard', tpl="ALL", update=None,
+                     overwrite=True):
         """Reducing the STD files after they have been obtained
         Running the muse_standard routine
 
@@ -627,13 +626,15 @@ class PipePrep(SofPipe) :
             self.recipe_std(self.current_sof, dir_std, name_std, mytpl)
 
         # Write the MASTER files Table and save it
-        self.save_expo_table("STD", std_table, "master", aggregate=False, update=update)
+        self.save_expo_table("STD", std_table, "master", aggregate=False,
+                             update=update, overwrite=overwrite)
 
         # Go back to original folder
         self.goto_prevfolder(addtolog=True)
 
     @print_my_function_name
-    def run_sky(self, sof_filename='sky', tpl="ALL", fraction=0.8, update=None):
+    def run_sky(self, sof_filename='sky', tpl="ALL", fraction=0.8,
+                update=None, overwrite=True):
         """Reducing the SKY after they have been scibasic reduced
         Will run the esorex muse_create_sky routine
 
@@ -677,7 +678,8 @@ class PipePrep(SofPipe) :
             self.recipe_sky(self.current_sof, dir_sky, name_sky, mytpl, iexpo, fraction)
 
         # Write the MASTER files Table and save it
-        self.save_expo_table("SKY", sky_table, "processed", aggregate=False, update=update)
+        self.save_expo_table("SKY", sky_table, "processed", aggregate=False,
+                             update=update, overwrite=overwrite)
 
         # Go back to original folder
         self.goto_prevfolder(addtolog=True)
@@ -742,7 +744,7 @@ class PipePrep(SofPipe) :
                     **extra_kwargs)
 
     @print_my_function_name
-    def run_check_align(self, offset_table_name, sof_filename='scipost', expotype="OBJECT", tpl="ALL", 
+    def run_check_align(self, name_offset_table, sof_filename='scipost', expotype="OBJECT", tpl="ALL",
             line=None, suffix="", folder_offset_table=None, **extra_kwargs):
         """Launch the scipost command to get individual exposures in a narrow
         band filter to check if the alignments are ok (after rotation 
@@ -780,7 +782,7 @@ class PipePrep(SofPipe) :
                     tpl=mytpl, list_expo=[iexpo], suffix=suffix, 
                     lambdaminmax=[lmin, lmax], save='cube', 
                     offset_list=True, filter_list=filter_for_alignment,
-                    offset_table_name=offset_table_name,
+                    name_offset_table=name_offset_table,
                     folder_offset_table=folder_offset_table,
                     **extra_kwargs)
 
@@ -895,7 +897,7 @@ class PipePrep(SofPipe) :
         return found_expo, list_expo, group_list_expo, group_table
 
     def _normalise_skycontinuum(self, mjd_expo, tpl_expo, iexpo, 
-            suffix="", offset_table_name=None, **kwargs):
+            suffix="", name_offset_table=None, **kwargs):
         """Create a normalised continuum, to be included in the sof file
         """
         stage = "processed"
@@ -924,11 +926,11 @@ class PipePrep(SofPipe) :
         if normalise_factor is None:
             # Checking input offset table and corresponding pixtables
             folder_offset_table = kwargs.pop("folder_offset_table", None)
-            self._read_offset_table(offset_table_name, folder_offset_table)
+            self._read_offset_table(name_offset_table, folder_offset_table)
             # Get the background value
             if mjd_names['table'] not in self.offset_table.columns:
                 upipe.print_warning("No MJD column in offset table {0}".format(
-                                     offset_table_name))
+                                     name_offset_table))
                 status = -1
             else:    
                 table_mjdobs = self.offset_table[mjd_names['table']]
@@ -941,7 +943,7 @@ class PipePrep(SofPipe) :
             if status < 0:
                 dict_err = {-1: "MJD", -2: "BACKGROUND"}
                 upipe.print_error("Table {0} - {1}".format(folder_offset_table,
-                                  offset_table_name))
+                                  name_offset_table))
                 upipe.print_error("Could not find {0} value in offset table".format(
                                     dict_err[status]))
                 upipe.print_warning("A background of 0 will be assumed, and")
@@ -957,7 +959,7 @@ class PipePrep(SofPipe) :
             # Integration of the continuum in that filter
             thisfile_musemode = expo_table[0]['mode']
             upipe.print_info("For this sky continuum, we use MUSE Mode = {0}".format(thisfile_musemode))
-            mycont.integrate(mymusefilter, AO_mask=("-AO-" in thisfile_musemode))
+            mycont.integrate(mymusefilter, ao_mask=("-AO-" in thisfile_musemode))
             mycont.get_normfactor(background, filter_for_alignment)
             normalise_factor = getattr(mycont, filter_for_alignment).norm
 
@@ -1002,7 +1004,7 @@ class PipePrep(SofPipe) :
             Type of skymethod. See MUSE manual.
         offset_list: bool
             If True, using an OFFSET list. Default is True.
-        offset_table_name: str
+        name_offset_table: str
             Name of the offset table table. If not provided, will use the 
             default name produced during the pipeline run.
         filter_for_alignment: str
@@ -1041,11 +1043,11 @@ class PipePrep(SofPipe) :
 
         # Offsets
         offset_list = kwargs.pop("offset_list", False)
-        offset_table_name = kwargs.pop("offset_table_name", None)
+        name_offset_table = kwargs.pop("name_offset_table", None)
         folder_offset_table = kwargs.pop("folder_offset_table", None)
         if offset_list:
             upipe.print_info("Will use offset table: {0} in {1}".format(
-                offset_table_name, folder_offset_table))
+                name_offset_table, folder_offset_table))
 
         # Misc parameters - autocalib, barycentric correction, AC
         autocalib = kwargs.pop("autocalib", "none")
@@ -1101,16 +1103,16 @@ class PipePrep(SofPipe) :
             if ref_wcs is not None:
                 self._sofdict['OUTPUT_WCS'] = [joinpath(folder_ref_wcs, ref_wcs)]
 
-            if offset_table_name is None:
+            if name_offset_table is None:
                 folder_offset_table = self._get_fullpath_expo(expotype, "processed")
-                offset_table_name = '{0}{1}_{2}_{3}.fits'.format(
+                name_offset_table = '{0}{1}_{2}_{3}.fits'.format(
                                        dict_files_products['ALIGN'][0],
                                        suffix, filter_for_alignment, tpl)
             else:
                 if folder_offset_table is None:
                     folder_offset_table = self.paths.alignment
             if offset_list :
-                self._sofdict['OFFSET_LIST'] = [joinpath(folder_offset_table, offset_table_name)]
+                self._sofdict['OFFSET_LIST'] = [joinpath(folder_offset_table, name_offset_table)]
 
             # The sky subtraction method on the sky continuum to normalise it
             # But only if requested
@@ -1123,7 +1125,7 @@ class PipePrep(SofPipe) :
                         tpl_expo=tpl, iexpo=list_group_expo[0], 
                         suffix=suffix_skycontinuum, 
                         folder_offset_table=folder_offset_table,
-                        offset_table_name=offset_table_name)
+                        name_offset_table=name_offset_table)
             else:
                 prefix_skycontinuum = ""
             if skymethod != "none":
@@ -1166,7 +1168,7 @@ class PipePrep(SofPipe) :
             self.save_expo_table(expotype, scipost_table, "reduced", 
                     "IMAGES_FOV{0}_{1}{2}_{3}_list_table.fits".format(
                         suffix, expotype, suffix_expo, tpl), 
-                        aggregate=False, update=True)
+                        aggregate=False, overwrite=True)
 
         # Go back to original folder
         self.goto_prevfolder(addtolog=True)
@@ -1249,7 +1251,7 @@ class PipePrep(SofPipe) :
         self.goto_prevfolder(addtolog=True)
         
     @print_my_function_name
-    def run_align_bypointing(self, sof_filename='exp_align_bypointing', expotype="OBJECT", 
+    def run_align_bydataset(self, sof_filename='exp_align_bydataset', expotype="OBJECT",
             list_expo=[], stage="processed", line=None, suffix="",
             tpl="ALL", **kwargs):
         """Aligning the individual exposures from a dataset
@@ -1271,10 +1273,10 @@ class PipePrep(SofPipe) :
         # Stop the process if only 1 or fewer exposures are retrieved
         if len(align_table) <= 1:
             if self.verbose:
-                upipe.print_warning("Pointing = {0}".format(self.pointing),
+                upipe.print_warning("Dataset = {0}".format(self.dataset),
                         pipe=self)
                 upipe.print_warning("No derived OFFSET LIST as only "
-                                    "1 exposure retrieved in this Pointing",
+                                    "1 exposure retrieved in this Dataset",
                         pipe=self)
             return
         
@@ -1286,8 +1288,8 @@ class PipePrep(SofPipe) :
         if line is not None:
             suffix += "_{0}".format(line)
 
-        # Use the pointing as a suffix for the names
-        pointing = "P{0:02d}".format(self.pointing)
+        # Use the dataset as a suffix for the names
+        obname = self._get_obname()
 
         # Now creating the SOF file, first reseting it
         self._sofdict.clear()
@@ -1306,33 +1308,33 @@ class PipePrep(SofPipe) :
 
         # Write the SOF
         self.write_sof(sof_filename=sof_filename + "{0}_{1}_{2}_{3}".format(
-                       long_suffix, expotype, pointing, tpl), new=True)
+                       long_suffix, expotype, obname, tpl), new=True)
         dir_align = self._get_fullpath_expo('OBJECT', "processed")
         namein_align = deepcopy(dict_files_products['ALIGN'])
         nameout_align = [name + "{0}_{1}_{2}_{3}".format(
-                         long_suffix, expotype, pointing, tpl) 
+                         long_suffix, expotype, obname, tpl)
                              for name in deepcopy(dict_files_products['ALIGN'])]
         for iter_file in dict_files_iexpo_products['ALIGN']:
             for i, row in enumerate(align_table):
                 namein_align.append('{0}_{1:04d}'.format(iter_file, i+1))
-                nameout_align.append('{0}_bypointing{1}_{2}_{3:04d}'.format(
+                nameout_align.append('{0}_bydataset{1}_{2}_{3:04d}'.format(
                     iter_file, long_suffix, row['tpls'], row['iexpo']))
 
-        # Find the alignment - Note that Pointing is used and tpl reflects the given selection
+        # Find the alignment - Note that Field is used and tpl reflects the given selection
         self.recipe_align(self.current_sof, dir_align, namein_align, nameout_align, 
-                tpl, pointing, **kwargs)
+                tpl, obname, **kwargs)
 
         # Write the MASTER files Table and save it
         self.save_expo_table(expotype, align_table, "reduced", 
-                "ALIGNED_IMAGES_BYPOINTING_{0}{1}_{2}_{3}_list_table.fits".format(expotype, 
-                long_suffix, pointing, tpl), aggregate=False, update=True)
+                "ALIGNED_IMAGES_BYDATASET_{0}{1}_{2}_{3}_list_table.fits".format(expotype,
+                long_suffix, obname, tpl), aggregate=False, update=True)
         
         # Go back to original folder
         self.goto_prevfolder(addtolog=True)
 
     @print_my_function_name
     def save_fine_alignment(self, name_offset_table=None):
-        """ Save the fine pointing alignment
+        """ Save the fine dataset alignment
         """
         if name_offset_table is None:
             name_offset_table = "XX"
@@ -1342,9 +1344,9 @@ class PipePrep(SofPipe) :
         self.dict_alignments[tstamp].save(name_offset_table)
 
     @print_my_function_name
-    def run_fine_alignment(self, name_ima_reference=None, nexpo=1, list_expo=[], line=None, 
-            bygroup=False, reset=False):
-        """Run the alignment on this pointing using or not a reference image
+    def run_fine_alignment(self, name_ima_reference=None, nexpo=1, list_expo=[],
+                           line=None, bygroup=False, reset=False, **kwargs):
+        """Run the alignment on this dataset using or not a reference image
         """
         # If not yet initialised, build the dictionary
         if not hasattr(self, "dict_alignments"):
@@ -1372,7 +1374,7 @@ class PipePrep(SofPipe) :
         
     @print_my_function_name
     def get_align_group(self, name_ima_reference=None, list_expo=[], line=None, 
-            suffix="", bygroup=False):
+            suffix="", bygroup=False, **kwargs):
         """Extract the needed information for a set of exposures to be aligned
         """
         # Selecting the table with the right iexpo
@@ -1400,7 +1402,8 @@ class PipePrep(SofPipe) :
                 list_names_muse = [joinpath(self._get_fullpath_expo("OBJECT", "processed"),
                     'IMAGE_FOV{0}_{1:04d}.fits'.format(long_suffix, iexpo)) 
                     for iexpo in list_group_expo]
-                self.align_group.append(AlignMusePointing(name_ima_reference, list_names_muse, flag="mytpl"))
+                self.align_group.append(AlignMuseField(name_ima_reference, list_names_muse,
+                                                   flag="mytpl"))
         # If not by group
         else:
             long_suffix = "{0}_{1}".format(suffix, filter_for_alignment)
@@ -1410,17 +1413,17 @@ class PipePrep(SofPipe) :
             # Giving a reference image name, doing the alignment w.r.t. to that specific image
             if name_ima_reference is None:
                 upipe.print_warning("Using the first MUSE exposure as a reference")
-                self.align_group.append(AlignMusePointing(list_names_muse[0], 
+                self.align_group.append(AlignMuseField(list_names_muse[0],
                     list_names_muse[1:], flag="mytpl"))
             else:
-                self.align_group.append(AlignMusePointing(name_ima_reference, 
+                self.align_group.append(AlignMuseField(name_ima_reference,
                     list_names_muse, flag="mytpl"))
 
     @print_my_function_name
-    def run_combine_pointing(self, sof_filename='exp_combine', expotype="OBJECT", 
+    def run_combine_dataset(self, sof_filename='exp_combine', expotype="OBJECT",
             list_expo=[], stage="processed", tpl="ALL", 
             lambdaminmax=[4000.,10000.], suffix="", **kwargs):
-        """Produce a cube from all frames in the pointing
+        """Produce a cube from all frames in the dataset
         list_expo or tpl specific arguments can still reduce the selection if needed
         """
         # Selecting the table with the right iexpo
@@ -1435,7 +1438,7 @@ class PipePrep(SofPipe) :
         # exp_combine needs a minimum of 2
         if len(list_expo) <= 1:
             if self.verbose:
-                upipe.print_warning("The combined pointing has only one exposure: process aborted",
+                upipe.print_warning("The combined dataset has only one exposure: process aborted",
                         pipe=self)
             return
 
@@ -1447,8 +1450,8 @@ class PipePrep(SofPipe) :
         filter_list = kwargs.pop("filter_list", self.filter_list)
         prefix_all = kwargs.pop("prefix_all", "")
 
-        # Use the pointing as a suffix for the names
-        pointing = "P{0:02d}".format(self.pointing)
+        # Use the dataset as a suffix for the names
+        obname = self._get_obname()
         # Save option
         save = kwargs.pop("save", "cube,combined")
 
@@ -1465,12 +1468,12 @@ class PipePrep(SofPipe) :
         offset_list = kwargs.pop("offset_list", True)
         folder_expo = self._get_fullpath_expo(expotype, stage)
         long_suffix = "{0}_{1}".format(suffix, filter_for_alignment)
-        if offset_list :
+        if offset_list:
             offset_list_tablename = kwargs.pop("offset_list_tablename", None)
             if offset_list_tablename is None:
                 offset_list_tablename = "{0}{1}_{2}_{3}_{4}.fits".format(
                         dict_files_products['ALIGN'][0], long_suffix,
-                        expotype, pointing, tpl)
+                        expotype, obname, tpl)
             if not os.path.isfile(joinpath(folder_expo, offset_list_tablename)):
                 upipe.print_error("OFFSET_LIST table {0} not found in folder {1}".format(
                         offset_list_tablename, folder_expo), pipe=self)
@@ -1497,7 +1500,6 @@ class PipePrep(SofPipe) :
                 suffix_prefinalnames=suffix_prefinalnames,
                 prefix_products=prefix_products,
                 lambdamin=lambdamin, lambdamax=lambdamax,
-                filter_for_alignment=filter_for_alignment,
                 save=save, suffix=suffix, filter_list=filter_list, **kwargs)
 
         # Go back to original folder
